"""
Quelques fonctions d'initialisation pour tests unitaires
"""

from tests.unit import sco_fake_gen
from app import db
from app import models

from app.scodoc import sco_codes_parcours


def build_formation_test(nb_mods=1):
    G = sco_fake_gen.ScoFake(verbose=False)
    _f = G.create_formation(
        acronyme="F3",
        titre="Formation 2",
        titre_officiel="Titre officiel 2",
        type_parcours=sco_codes_parcours.ParcoursBUT.TYPE_PARCOURS,
    )
    _ue1 = G.create_ue(
        formation_id=_f["formation_id"], acronyme="UE1", titre="ue 1", semestre_idx=2
    )
    _ue2 = G.create_ue(
        formation_id=_f["formation_id"], acronyme="UE2", titre="ue 2", semestre_idx=2
    )
    _ue3 = G.create_ue(
        formation_id=_f["formation_id"], acronyme="UE3", titre="ue 3", semestre_idx=2
    )
    # une 4eme UE en dehors du semestre 2
    _ = G.create_ue(
        formation_id=_f["formation_id"], acronyme="UE41", titre="ue 41", semestre_idx=4
    )
    _mat = G.create_matiere(ue_id=_ue1["ue_id"], titre="matière test")
    module_ids = []
    for i in range(nb_mods):
        mod = G.create_module(
            matiere_id=_mat["matiere_id"],
            code=f"TSM{i}",
            coefficient=1.0,
            titre=f"module test {i}",
            ue_id=_ue1["ue_id"],
            formation_id=_f["formation_id"],
            semestre_id=2,
        )
        module_ids.append(mod["id"])
    return G, _f["id"], _ue1["id"], _ue2["id"], _ue3["id"], module_ids


def build_modules_with_evaluations(
    ue_coefs=(1.0, 2.0, 3.0), nb_mods=1, nb_evals_per_modimpl=1
):
    """Utilisé dans plusieurs tests:
    - création formation 3 UE, 1 module
    - 1 semestre, nb_mod moduleimpl, 1 eval/moduleimpl
    """
    nb_ues = len(ue_coefs)  # 3 UEs par défaut dans ce test
    G, formation_id, ue1_id, ue2_id, ue3_id, module_ids = build_formation_test(
        nb_mods=nb_mods
    )
    ue1 = models.UniteEns.query.get(ue1_id)
    ue2 = models.UniteEns.query.get(ue2_id)
    ue3 = models.UniteEns.query.get(ue3_id)
    for module_id in module_ids:
        mod = models.Module.query.get(module_id)
        # Coef du module vers les UE
        c1, c2, c3 = ue_coefs
        coefs_mod = {ue1.id: c1, ue2.id: c2, ue3.id: c3}
        mod.set_ue_coef_dict(coefs_mod)
        assert mod.get_ue_coef_dict() == coefs_mod
    # Mise en place:
    sem = G.create_formsemestre(
        formation_id=formation_id,
        semestre_id=2,
        date_debut="01/01/2021",
        date_fin="30/06/2021",
    )
    evaluation_ids = []
    for module_id in module_ids:
        mi = G.create_moduleimpl(
            module_id=module_id,
            formsemestre_id=sem["formsemestre_id"],
        )
        moduleimpl_id = mi["id"]
        modimpl = models.ModuleImpl.query.get(moduleimpl_id)
        assert modimpl.formsemestre.formation.get_parcours().APC_SAE  # BUT
        # Check ModuleImpl
        ues = modimpl.formsemestre.query_ues().all()
        assert len(ues) == 3
        #
        for _ in range(nb_evals_per_modimpl):
            e = G.create_evaluation(
                moduleimpl_id=moduleimpl_id,
                jour="01/01/2021",
                description="evaluation 1",
                coefficient=0,
            )
            evaluation_ids.append(e["evaluation_id"])
    return G, formation_id, sem, evaluation_ids, ue1, ue2, ue3
